@@ -18,6 +18,7 @@
  */
 
 #include <gtk/gtk.h>
+#include <stdio.h>
 #include "gailclist.h"
 #include "gailclistcell.h"
 #include "gailcellparent.h"
@@ -261,51 +262,51 @@ gail_clist_get_type (void)
   static GType type = 0;
 
   if (!type)
-  {
-    static const GTypeInfo tinfo =
     {
-      sizeof (GailCListClass),
-      (GBaseInitFunc) NULL, /* base init */
-      (GBaseFinalizeFunc) NULL, /* base finalize */
-      (GClassInitFunc) gail_clist_class_init, /* class init */
-      (GClassFinalizeFunc) NULL, /* class finalize */
-      NULL, /* class data */
-      sizeof (GailCList), /* instance size */
-      0, /* nb preallocs */
-      (GInstanceInitFunc) NULL, /* instance init */
-      NULL /* value table */
-    };
+      static const GTypeInfo tinfo =
+      {
+        sizeof (GailCListClass),
+        (GBaseInitFunc) NULL, /* base init */
+        (GBaseFinalizeFunc) NULL, /* base finalize */
+        (GClassInitFunc) gail_clist_class_init, /* class init */
+        (GClassFinalizeFunc) NULL, /* class finalize */
+        NULL, /* class data */
+        sizeof (GailCList), /* instance size */
+        0, /* nb preallocs */
+        (GInstanceInitFunc) NULL, /* instance init */
+        NULL /* value table */
+      };
 
-    static const GInterfaceInfo atk_table_info =
-    {
+      static const GInterfaceInfo atk_table_info =
+      {
         (GInterfaceInitFunc) atk_table_interface_init,
         (GInterfaceFinalizeFunc) NULL,
         NULL
-    };
+      };
 
-    static const GInterfaceInfo atk_selection_info =
-    {
+      static const GInterfaceInfo atk_selection_info =
+      {
         (GInterfaceInitFunc) atk_selection_interface_init,
         (GInterfaceFinalizeFunc) NULL,
         NULL
-    };
+      };
 
-    static const GInterfaceInfo gail_cell_parent_info =
-    {
+      static const GInterfaceInfo gail_cell_parent_info =
+      {
         (GInterfaceInitFunc) gail_cell_parent_interface_init,
         (GInterfaceFinalizeFunc) NULL,
         NULL
-    };
+      };
 
-    type = g_type_register_static (GAIL_TYPE_CONTAINER,
-                                   "GailCList", &tinfo, 0);
-    g_type_add_interface_static (type, ATK_TYPE_TABLE,
-                                 &atk_table_info);
-    g_type_add_interface_static (type, ATK_TYPE_SELECTION,
-                                 &atk_selection_info);
-    g_type_add_interface_static (type, GAIL_TYPE_CELL_PARENT,
-                                 &gail_cell_parent_info);
-  }
+      type = g_type_register_static (GAIL_TYPE_CONTAINER,
+                                     "GailCList", &tinfo, 0);
+      g_type_add_interface_static (type, ATK_TYPE_TABLE,
+                                   &atk_table_info);
+      g_type_add_interface_static (type, ATK_TYPE_SELECTION,
+                                   &atk_selection_info);
+      g_type_add_interface_static (type, GAIL_TYPE_CELL_PARENT,
+                                   &gail_cell_parent_info);
+    }
   return type;
 }
 
@@ -370,10 +371,10 @@ gail_clist_real_init (GailWidget         *widget,
   clist->n_cols = gtk_clist->columns;
   clist->columns = g_new (GailCListColumn, gtk_clist->columns);
   for (i = 0; i < gtk_clist->columns; i++)
-  {
-    clist->columns[i].description = NULL;
-    clist->columns[i].header = NULL;
-  }
+    {
+      clist->columns[i].description = NULL;
+      clist->columns[i].header = NULL;
+    }
   /*
    * Set up signal handlers for select-row and unselect-row
    */
@@ -411,39 +412,39 @@ gail_clist_finalize (GObject            *object)
     g_object_unref (clist->summary);
 
   for (i = 0; i < clist->n_cols; i++)
-  {
-    g_free (clist->columns[i].description);
-    if (clist->columns[i].header)
-      g_object_unref (clist->columns[i].header);
-  }
+    {
+      g_free (clist->columns[i].description);
+      if (clist->columns[i].header)
+        g_object_unref (clist->columns[i].header);
+    }
   g_free (clist->columns);
 
   array = clist->row_data;
 
   if (array)
-  {
-    for (i = 0; i < array->len; i++)
     {
-      GailCListRow *row_data;
+      for (i = 0; i < array->len; i++)
+        {
+          GailCListRow *row_data;
 
-      row_data = g_array_index (array, GailCListRow*, i);
+          row_data = g_array_index (array, GailCListRow*, i);
 
-      if (row_data->header)
-        g_object_unref (row_data->header);
-      g_free (row_data->description);
+          if (row_data->header)
+            g_object_unref (row_data->header);
+          g_free (row_data->description);
+        }
     }
-  }
 
   if (clist->cell_data)
-  {
-    GList *temp_list;
-
-    for (temp_list = clist->cell_data; temp_list; temp_list = temp_list->next)
     {
-      g_list_free (temp_list->data);
+      GList *temp_list;
+
+      for (temp_list = clist->cell_data; temp_list; temp_list = temp_list->next)
+        {
+          g_list_free (temp_list->data);
+        }
+      g_list_free (clist->cell_data);
     }
-    g_list_free (clist->cell_data);
-  }
 }
 
 static gint
@@ -456,12 +457,10 @@ gail_clist_get_n_children (AtkObject *obj)
 
   widget = GTK_ACCESSIBLE (obj)->widget;
   if (widget == NULL)
-  {
     /*
      * State is defunct
      */
     return 0;
-  }
 
   row = gail_clist_get_n_rows (ATK_TABLE (obj));
   col = gail_clist_get_n_actual_columns (GTK_CLIST (widget));
@@ -481,12 +480,10 @@ gail_clist_ref_child (AtkObject *obj,
 
   widget = GTK_ACCESSIBLE (obj)->widget;
   if (widget == NULL)
-  {
     /*
      * State is defunct
      */
     return NULL;
-  }
 
   n_columns = gail_clist_get_n_actual_columns (GTK_CLIST (widget));
   if (!n_columns)
@@ -548,10 +545,8 @@ gail_clist_clear_selection (AtkSelection   *selection)
   
   widget = GTK_ACCESSIBLE (selection)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return FALSE;
-  }
   
   clist = GTK_CLIST (widget);
   gtk_clist_unselect_all(clist);
@@ -576,6 +571,7 @@ gail_clist_ref_selection (AtkSelection   *selection,
   g_free (selected_rows);
   selected_column = gail_clist_get_actual_column (ATK_TABLE (selection), 
                                                   i % visible_columns);
+
   return gail_clist_ref_at (ATK_TABLE (selection), selected_row, 
                             selected_column);
 }
@@ -586,14 +582,13 @@ gail_clist_get_selection_count (AtkSelection   *selection)
   gint n_rows_selected;
 
   n_rows_selected = gail_clist_get_selected_rows (ATK_TABLE (selection), NULL);
+
   if (n_rows_selected > 0)
-  {
     /*
      * The number of cells selected is the number of columns
      * times the number of selected rows
      */
     return gail_clist_get_n_columns (ATK_TABLE (selection)) * n_rows_selected;
-  }
   return 0;
 }
 
@@ -619,10 +614,9 @@ gail_clist_select_all_selection (AtkSelection   *selection)
   
   widget = GTK_ACCESSIBLE (selection)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return FALSE;
-  }
+
   clist = GTK_CLIST (widget);
   gtk_clist_select_all(clist);
 
@@ -639,10 +633,9 @@ gail_clist_ref_at (AtkTable *table,
 
   widget = GTK_ACCESSIBLE (table)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return NULL;
-  }
+
   actual_column = gail_clist_get_actual_column (table, column);
   return gail_clist_ref_at_actual (table, row, actual_column);
 }
@@ -664,13 +657,14 @@ gail_clist_ref_at_actual (AtkTable      *table,
   AtkObject *return_object;
   gint n_rows, n_columns;
   GailCell *cell;
+
+  g_return_val_if_fail (GTK_IS_ACCESSIBLE (table), NULL);
   
   widget = GTK_ACCESSIBLE (table)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return NULL;
-  }
+
   clist = GTK_CLIST (widget);
   n_rows = gail_clist_get_n_rows (table); 
   n_columns = gail_clist_get_n_actual_columns (clist); 
@@ -685,13 +679,13 @@ gail_clist_ref_at_actual (AtkTable      *table,
    */
   cell = gail_clist_find_cell (GAIL_CLIST (table), row, column);
   if (cell)
-  {
-    g_object_ref (cell);
-    return ATK_OBJECT (cell);
-  }
+    {
+      g_object_ref (cell);
+      return ATK_OBJECT (cell);
+    }
   cellType = gtk_clist_get_cell_type(clist, row, column);
   switch (cellType) 
-  {
+    {
     case GTK_CELL_TEXT:
       return_object = gail_clist_cell_new ();
       break;
@@ -705,36 +699,38 @@ gail_clist_ref_at_actual (AtkTable      *table,
       /* Don't handle GTK_CELL_EMPTY or GTK_CELL_WIDGET, return NULL */
       return_object = NULL;
       break;
-  }
-  if (return_object)
-  {
-    cell = GAIL_CELL (return_object);
-
-    gail_cell_init (cell, widget, ATK_OBJECT (table),
-                    row, column,
-                    column + row * gail_clist_get_n_actual_columns (clist));
-    /*
-     * Store the cell in a cache
-     */
-    gail_clist_cell_data_new (GAIL_CLIST (table), cell, column, row);
-    /*
-     * If the column is visible, sets the cell's state
-     */
-    if (clist->column[column].visible)
-    {
-      GdkRectangle cell_rect, visible_rect;
-
-      gail_clist_get_cell_area (GAIL_CELL_PARENT (table), cell, &cell_rect);
-      gail_clist_get_visible_rect (clist, &visible_rect);
-      if (gail_clist_is_cell_visible (&cell_rect, &visible_rect))
-        gail_cell_add_state (cell, ATK_STATE_VISIBLE, FALSE);
     }
-    /*
-     * If a row is selected, all cells in the row are selected
-     */
-    if (gail_clist_is_row_selected (table, row))
-      gail_cell_add_state (cell, ATK_STATE_SELECTED, FALSE);
-  }
+  if (return_object)
+    {
+      cell = GAIL_CELL (return_object);
+
+      g_return_val_if_fail (ATK_IS_OBJECT (table), NULL);
+
+      gail_cell_init (cell, widget, ATK_OBJECT (table),
+                      row, column,
+                      column + row * gail_clist_get_n_actual_columns (clist));
+      /*
+       * Store the cell in a cache
+       */
+      gail_clist_cell_data_new (GAIL_CLIST (table), cell, column, row);
+      /*
+       * If the column is visible, sets the cell's state
+       */
+      if (clist->column[column].visible)
+        {
+          GdkRectangle cell_rect, visible_rect;
+  
+          gail_clist_get_cell_area (GAIL_CELL_PARENT (table), cell, &cell_rect);
+          gail_clist_get_visible_rect (clist, &visible_rect);
+          if (gail_clist_is_cell_visible (&cell_rect, &visible_rect))
+            gail_cell_add_state (cell, ATK_STATE_VISIBLE, FALSE);
+        }
+      /*
+       * If a row is selected, all cells in the row are selected
+       */
+      if (gail_clist_is_row_selected (table, row))
+        gail_cell_add_state (cell, ATK_STATE_SELECTED, FALSE);
+    }
 
   return return_object; 
 }
@@ -755,10 +751,9 @@ gail_clist_get_n_columns (AtkTable      *table)
 
   widget = GTK_ACCESSIBLE (table)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return 0;
-  }
+
   clist = GTK_CLIST (widget);
 
   return gail_clist_get_visible_column (table, 
@@ -830,10 +825,9 @@ gail_clist_get_n_rows (AtkTable      *table)
 
   widget = GTK_ACCESSIBLE (table)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return 0;
-  }
+
   clist = GTK_CLIST (widget);
   return clist->rows;
 }
@@ -879,10 +873,9 @@ gail_clist_add_row_selection (AtkTable      *table,
 
   widget = GTK_ACCESSIBLE (table)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return 0;
-  }
+
   clist = GTK_CLIST (widget);
   gtk_clist_select_row (clist, row, -1);
   if (gail_clist_is_row_selected (table, row))
@@ -900,10 +893,9 @@ gail_clist_remove_row_selection (AtkTable      *table,
 
   widget = GTK_ACCESSIBLE (table)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return 0;
-  }
+
   clist = GTK_CLIST (widget);
   if (gail_clist_is_row_selected (table, row))
   {
@@ -925,10 +917,9 @@ gail_clist_get_selected_rows (AtkTable *table,
 
   widget = GTK_ACCESSIBLE (table)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return 0;
-  }
+
   clist = GTK_CLIST (widget);
  
   n_selected = g_list_length (clist->selection);
@@ -937,20 +928,20 @@ gail_clist_get_selected_rows (AtkTable *table,
     return 0;
 
   if (rows_selected)
-  {
-    gint *selected_rows;
-
-    selected_rows = (gint*) g_malloc (sizeof (gint) * n_selected);
-    list = clist->selection;
-
-    i = 0;
-    while (list)
     {
-      selected_rows[i++] = GPOINTER_TO_INT (list->data);
-      list = list->next;
+      gint *selected_rows;
+
+      selected_rows = (gint*) g_malloc (sizeof (gint) * n_selected);
+      list = clist->selection;
+
+      i = 0;
+      while (list)
+        {
+          selected_rows[i++] = GPOINTER_TO_INT (list->data);
+          list = list->next;
+        }
+      *rows_selected = selected_rows;
     }
-    *rows_selected = selected_rows;
-  }
   return n_selected;
 }
 
@@ -964,10 +955,9 @@ gail_clist_is_row_selected (AtkTable      *table,
 
   widget = GTK_ACCESSIBLE (table)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
 	return FALSE;
-  }
+
   clist = GTK_CLIST (widget);
       
   if (row < 0 || row >= clist->rows)
@@ -991,22 +981,23 @@ gail_clist_set_caption (AtkTable      *table,
                         AtkObject     *caption)
 {
   GailCList* obj = GAIL_CLIST (table);
-  AtkPropertyValues values;
+  AtkPropertyValues values = { 0, };
   AtkObject *old_caption;
 
   old_caption = obj->caption;
   obj->caption = caption;
   if (obj->caption)
     g_object_ref (obj->caption);
-  memset (&values.old_value, 0, sizeof (GValue));
-  memset (&values.new_value, 0, sizeof (GValue));
+
   g_value_init (&values.old_value, G_TYPE_POINTER);
   g_value_set_pointer (&values.old_value, old_caption);
   g_value_init (&values.new_value, G_TYPE_POINTER);
   g_value_set_pointer (&values.new_value, obj->caption);
 
-  values.property_name = "accessible-table-caption";
-  g_signal_emit_by_name (table, "property-change", &values, NULL);
+  values.property_name = "accessible_table_caption";
+  g_signal_emit_by_name (table, 
+                         "property_change::accessible_table_caption", 
+                         &values, NULL);
   if (old_caption)
     g_object_unref (old_caption);
 }
@@ -1017,7 +1008,7 @@ gail_clist_set_column_description (AtkTable      *table,
                                    const gchar   *description)
 {
   GailCList *clist = GAIL_CLIST (table);
-  AtkPropertyValues values;
+  AtkPropertyValues values = { 0, };
   gint actual_column;
 
   if (column < 0 || column >= gail_clist_get_n_columns (table))
@@ -1029,13 +1020,14 @@ gail_clist_set_column_description (AtkTable      *table,
   actual_column = gail_clist_get_actual_column (table, column);
   g_free (clist->columns[actual_column].description);
   clist->columns[actual_column].description = g_strdup (description);
-  memset (&values.old_value, 0, sizeof (GValue));
-  memset (&values.new_value, 0, sizeof (GValue));
+
   g_value_init (&values.new_value, G_TYPE_INT);
   g_value_set_int (&values.new_value, column);
 
-  values.property_name = "accessible-table-column-description";
-  g_signal_emit_by_name (table, "property-change", &values, NULL);
+  values.property_name = "accessible_table_column_description";
+  g_signal_emit_by_name (table, 
+                         "property_change::accessible_table_column_description",
+                          &values, NULL);
 
 }
 
@@ -1045,7 +1037,7 @@ gail_clist_set_column_header (AtkTable      *table,
                               AtkObject     *header)
 {
   GailCList *clist = GAIL_CLIST (table);
-  AtkPropertyValues values;
+  AtkPropertyValues values = { 0, };
   gint actual_column;
 
   if (column < 0 || column >= gail_clist_get_n_columns (table))
@@ -1057,13 +1049,14 @@ gail_clist_set_column_header (AtkTable      *table,
   if (header)
     g_object_ref (header);
   clist->columns[actual_column].header = header;
-  memset (&values.old_value, 0, sizeof (GValue));
-  memset (&values.new_value, 0, sizeof (GValue));
+
   g_value_init (&values.new_value, G_TYPE_INT);
   g_value_set_int (&values.new_value, column);
 
-  values.property_name = "accessible-table-column-header";
-  g_signal_emit_by_name (table, "property-change", &values, NULL);
+  values.property_name = "accessible_table_column_header";
+  g_signal_emit_by_name (table, 
+                         "property_change::accessible_table_column-header",
+                         &values, NULL);
 }
 
 static void
@@ -1087,22 +1080,23 @@ gail_clist_set_summary (AtkTable      *table,
                         AtkObject     *accessible)
 {
   GailCList* obj = GAIL_CLIST (table);
-  AtkPropertyValues values;
+  AtkPropertyValues values = { 0, };
   AtkObject *old_summary;
 
   old_summary = obj->summary;
   obj->summary = accessible;
   if (obj->summary)
     g_object_ref (obj->summary);
-  memset (&values.old_value, 0, sizeof (GValue));
-  memset (&values.new_value, 0, sizeof (GValue));
+
   g_value_init (&values.old_value, G_TYPE_POINTER);
   g_value_set_pointer (&values.old_value, old_summary);
   g_value_init (&values.new_value, G_TYPE_POINTER);
   g_value_set_pointer (&values.new_value, obj->summary);
 
-  values.property_name = "accessible-table-summary";
-  g_signal_emit_by_name (table, "property-change", &values, NULL);
+  values.property_name = "accessible_table_summary";
+  g_signal_emit_by_name (table, 
+                         "property_change::accessible_table_summary", 
+                         &values, NULL);
   if (old_summary)
     g_object_unref (old_summary);
 }
@@ -1145,15 +1139,15 @@ gail_clist_get_cell_extents (GailCellParent *parent,
   *height = cell_rect.height;
   gail_clist_get_visible_rect (clist, &visible_rect);
   if (gail_clist_is_cell_visible (&cell_rect, &visible_rect))
-  {
-    *x = cell_rect.x + widget_x;
-    *y = cell_rect.y + widget_y;
-  }
+    {
+      *x = cell_rect.x + widget_x;
+      *y = cell_rect.y + widget_y;
+    }
   else
-  {
-    *x = G_MININT;
-    *y = G_MININT;
-  }
+    {
+      *x = G_MININT;
+      *y = G_MININT;
+    }
 }
 
 static void
@@ -1188,21 +1182,19 @@ gail_clist_select_row_gtk (GtkCList *clist,
   gail_clist = GAIL_CLIST (data);
 
   for (temp_list = gail_clist->cell_data; temp_list; temp_list = temp_list->next)
-  {
-    GailCListCellData *cell_data;
-
-    cell_data = (GailCListCellData *) (temp_list->data);
-
-    if (row == cell_data->row_number)
     {
-      /*
-       * Row is selected
-       */
-      gail_cell_add_state (cell_data->gail_cell, ATK_STATE_SELECTED, TRUE);
-    }
-  }
+      GailCListCellData *cell_data;
 
-  g_object_notify (G_OBJECT (gail_clist), "accessible-selection");
+      cell_data = (GailCListCellData *) (temp_list->data);
+
+      if (row == cell_data->row_number)
+        /*
+         * Row is selected
+         */
+        gail_cell_add_state (cell_data->gail_cell, ATK_STATE_SELECTED, TRUE);
+    }
+
+  g_signal_emit_by_name (gail_clist, "selection_changed");
 }
 
 static void
@@ -1218,21 +1210,19 @@ gail_clist_unselect_row_gtk (GtkCList *clist,
   gail_clist = GAIL_CLIST (data);
 
   for (temp_list = gail_clist->cell_data; temp_list; temp_list = temp_list->next)
-  {
-    GailCListCellData *cell_data;
-
-    cell_data = (GailCListCellData *) (temp_list->data);
-
-    if (row == cell_data->row_number)
     {
-      /*
-       * Row is unselected
-       */
-      gail_cell_remove_state (cell_data->gail_cell, ATK_STATE_SELECTED, TRUE);
-    }
-  }
+      GailCListCellData *cell_data;
 
-  g_object_notify (G_OBJECT (gail_clist), "accessible-selection");
+      cell_data = (GailCListCellData *) (temp_list->data);
+
+      if (row == cell_data->row_number)
+        /*
+         * Row is unselected
+         */
+        gail_cell_remove_state (cell_data->gail_cell, ATK_STATE_SELECTED, TRUE);
+    }
+
+  g_signal_emit_by_name (gail_clist, "selection_changed");
 }
 
 /*
@@ -1250,10 +1240,9 @@ gail_clist_get_visible_column (AtkTable *table,
 
   widget = GTK_ACCESSIBLE (table)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return 0;
-  }
+
   clist = GTK_CLIST (widget);
   for (i = 0, vis_columns = 0; i < column; i++)
     if (clist->column[i].visible)
@@ -1273,20 +1262,19 @@ gail_clist_get_actual_column (AtkTable *table,
 
   widget = GTK_ACCESSIBLE (table)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return 0;
-  }
+
   clist = GTK_CLIST (widget);
   for (i = 0, vis_columns = 0; i < clist->columns; i++)
-  {
-    if (clist->column[i].visible)
     {
-      if (visible_column == vis_columns)
-        return i;
-      vis_columns++;
+      if (clist->column[i].visible)
+        {
+          if (visible_column == vis_columns)
+            return i;
+          vis_columns++;
+        }
     }
-  }
   return 0;  
 }
 
@@ -1304,14 +1292,14 @@ gail_clist_set_row_data (AtkTable      *table,
   GailCListRow* row_data;
   gint i;
   gboolean found = FALSE;
-  AtkPropertyValues values;
+  AtkPropertyValues values = { 0, };
+  gchar *signal_name;
 
   widget = GTK_ACCESSIBLE (table)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return;
-  }
+
   gtk_clist = GTK_CLIST (widget);
   if (row < 0 || row >= gtk_clist->rows)
     return;
@@ -1319,64 +1307,70 @@ gail_clist_set_row_data (AtkTable      *table,
   gail_clist = GAIL_CLIST (table);
 
   if (gail_clist->row_data == NULL)
-  {
-    gail_clist->row_data = g_array_sized_new (FALSE, TRUE, sizeof (GailCListRow *), 0);
-  }
+    gail_clist->row_data = g_array_sized_new (FALSE, TRUE, 
+                                              sizeof (GailCListRow *), 0);
 
   array = gail_clist->row_data;
 
   for (i = 0; i < array->len; i++)
-  {
-    row_data = g_array_index (array, GailCListRow*, i);
+    {
+      row_data = g_array_index (array, GailCListRow*, i);
 
-    if (row == row_data->row_number)
-    {
-      found = TRUE;
-      if (is_header)
-      {
-        if (row_data->header)
-          g_object_unref (row_data->header);
-        row_data->header = header;
-        if (row_data->header)
-          g_object_ref (row_data->header);
-      }
-      else
-      {
-        g_free (row_data->description);
-        row_data->description = g_strdup (row_data->description);
-      }
-      break;
-    }
-  } 
+      if (row == row_data->row_number)
+        {
+          found = TRUE;
+          if (is_header)
+            {
+              if (row_data->header)
+                g_object_unref (row_data->header);
+              row_data->header = header;
+              if (row_data->header)
+                g_object_ref (row_data->header);
+            }
+          else
+            {
+              g_free (row_data->description);
+              row_data->description = g_strdup (row_data->description);
+            }
+          break;
+        }
+    } 
   if (found == FALSE)
-  {
-    row_data = g_new (GailCListRow, 1);
-    row_data->row_number = row;
-    row_data->row_data = ROW_ELEMENT (gtk_clist, row)->data;
-    if (is_header)
     {
-      row_data->header = header;
-      if (row_data->header)
-        g_object_ref (row_data->header);
-      row_data->description = NULL;
+      row_data = g_new (GailCListRow, 1);
+      row_data->row_number = row;
+      row_data->row_data = ROW_ELEMENT (gtk_clist, row)->data;
+      if (is_header)
+        {
+          row_data->header = header;
+          if (row_data->header)
+            g_object_ref (row_data->header);
+          row_data->description = NULL;
+        }
+      else
+        {
+          row_data->description = g_strdup (row_data->description);
+          row_data->header = NULL;
+        }
+      g_array_append_val (array, row_data);
     }
-    else
-    {
-      row_data->description = g_strdup (row_data->description);
-      row_data->header = NULL;
-    }
-    g_array_append_val (array, row_data);
-  }
-  memset (&values.old_value, 0, sizeof (GValue));
-  memset (&values.new_value, 0, sizeof (GValue));
+
   g_value_init (&values.new_value, G_TYPE_INT);
   g_value_set_int (&values.new_value, row);
 
   if (is_header)
-    values.property_name = "accessible-table-row-header";
+    {
+      values.property_name = "accessible_table_row_header";
+      signal_name = "property_change::accessible_table_row_header";
+    }
   else
-    values.property_name = "accessible-table-row-description";
-  g_signal_emit_by_name (table, "property-change", &values, NULL);
+    {
+      values.property_name = "accessible_table_row_description";
+      signal_name = "property_change::accessible_table_row_description";
+    }
+  g_signal_emit_by_name (table, 
+                         signal_name,
+                         &values, NULL);
 
 }
 
@@ -1393,10 +1387,9 @@ gail_clist_get_row_data (AtkTable      *table,
 
   widget = GTK_ACCESSIBLE (table)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return NULL;
-  }
+
   clist = GTK_CLIST (widget);
   if (row < 0 || row >= clist->rows)
     return NULL;
@@ -1409,14 +1402,13 @@ gail_clist_get_row_data (AtkTable      *table,
   array = obj->row_data;
 
   for (i = 0; i < array->len; i++)
-  {
-    row_data = g_array_index (array, GailCListRow*, i);
-
-    if (row == row_data->row_number)
     {
-      return row_data;
+      row_data = g_array_index (array, GailCListRow*, i);
+
+      if (row == row_data->row_number)
+        return row_data;
     }
-  }
+ 
   return NULL;
 }
 
@@ -1488,17 +1480,17 @@ gail_clist_cell_data_remove (GailCList *clist,
   GList *temp_list;
 
   for (temp_list = clist->cell_data; temp_list; temp_list = temp_list->next)
-  {
-    GailCListCellData *cell_data;
-
-    cell_data = (GailCListCellData *) temp_list->data;
-    if (cell_data->gail_cell == cell)
     {
-      clist->cell_data = g_list_remove_link (clist->cell_data, temp_list);
-      g_free (cell_data);
-      return;
+      GailCListCellData *cell_data;
+
+      cell_data = (GailCListCellData *) temp_list->data;
+      if (cell_data->gail_cell == cell)
+        {
+          clist->cell_data = g_list_remove_link (clist->cell_data, temp_list);
+          g_free (cell_data);
+          return;
+        }
     }
-  }
   g_warning ("No cell removed in gail_clist_cell_data_remove\n");
 }
 
@@ -1510,15 +1502,15 @@ gail_clist_find_cell (GailCList     *clist,
   GList *temp_list;
 
   for (temp_list = clist->cell_data; temp_list; temp_list = temp_list->next)
-  {
-    GailCListCellData *cell_data;
+    {
+      GailCListCellData *cell_data;
 
-    cell_data = (GailCListCellData *) (temp_list->data);
+      cell_data = (GailCListCellData *) (temp_list->data);
 
-    if ((cell_data->column_number == column) &&
-        (cell_data->row_number == row))
-      return cell_data->gail_cell;
-  }
+      if ((cell_data->column_number == column) &&
+          (cell_data->row_number == row))
+        return cell_data->gail_cell;
+    }
   return NULL;
 }
 
@@ -1544,24 +1536,20 @@ gail_clist_adjustment_changed (GtkAdjustment *adjustment,
   /* loop over the cells and report if they are visible or not. */
   /* Must loop through them all */
   for (temp_list = obj->cell_data; temp_list; temp_list = temp_list->next)
-  {
-    GailCell *cell;
-    GailCListCellData *cell_data;
-
-    cell_data = (GailCListCellData *) (temp_list->data);
-    cell = cell_data->gail_cell;
-
-    gail_clist_get_cell_area (GAIL_CELL_PARENT (atk_obj), 
-                              cell, &cell_rect);
-    if (gail_clist_is_cell_visible (&cell_rect, &visible_rect))
     {
-      gail_cell_add_state (cell, ATK_STATE_VISIBLE, TRUE);
+      GailCell *cell;
+      GailCListCellData *cell_data;
+
+      cell_data = (GailCListCellData *) (temp_list->data);
+      cell = cell_data->gail_cell;
+
+      gail_clist_get_cell_area (GAIL_CELL_PARENT (atk_obj), 
+                                cell, &cell_rect);
+      if (gail_clist_is_cell_visible (&cell_rect, &visible_rect))
+        gail_cell_add_state (cell, ATK_STATE_VISIBLE, TRUE);
+      else
+        gail_cell_remove_state (cell, ATK_STATE_VISIBLE, TRUE);
     }
-    else
-    {
-      gail_cell_remove_state (cell, ATK_STATE_VISIBLE, TRUE);
-    }
-  }
-  g_object_notify (G_OBJECT (atk_obj), "accessible-visible-data");
+  g_signal_emit_by_name (atk_obj, "visible_data_changed");
 }
 
