@@ -80,10 +80,13 @@ static gboolean	  gail_entry_set_selection	   (AtkText		 *text,
 						    gint		 start_offset,
 						    gint		 end_offset);
 static gint	  gail_entry_get_character_count   (AtkText		 *text);
-static AtkAttributeSet *  gail_entry_ref_run_attributes (AtkText	 *text,
+static AtkAttributeSet *  gail_entry_get_run_attributes 
+                                                   (AtkText              *text,
 						    gint		 offset,
         					    gint		 *start_offset,
 					       	    gint 		 *end_offset);
+static AtkAttributeSet *  gail_entry_get_default_attributes 
+                                                   (AtkText              *text);
 static void gail_entry_get_character_extents       (AtkText	         *text,
 						    gint 	         offset,
 		                                    gint 	         *x,
@@ -98,10 +101,6 @@ static gint gail_entry_get_offset_at_point         (AtkText              *text,
 /* atkeditabletext.h */
 
 static void       atk_editable_text_interface_init (AtkEditableTextIface *iface);
-static gboolean   gail_entry_set_run_attributes    (AtkEditableText     *text,
-                                                    AtkAttributeSet      *attrib_set,
-						    gint		 start_offset,
-						    gint 		 end_offset);
 static void       gail_entry_set_text_contents     (AtkEditableText      *text,
                                                     const gchar          *string);
 static void       gail_entry_insert_text           (AtkEditableText      *text,
@@ -134,6 +133,8 @@ static void	  _gail_entry_delete_text_cb	   (GtkEntry		 *entry,
                                                     gint		 arg1,
                                                     gint		 arg2);
 static void	  _gail_entry_changed_cb           (GtkEntry		 *entry);
+static gboolean    check_for_selection_change       (GailEntry           *entry,
+                                                     GtkEntry            *gail_entry);
 
 static GailWidgetClass *parent_class = NULL;
 
@@ -151,44 +152,43 @@ gail_entry_get_type (void)
   static GType type = 0;
 
   if (!type)
-  {
-    static const GTypeInfo tinfo =
     {
-      sizeof (GailEntryClass),
-      (GBaseInitFunc) NULL, /* base init */
-      (GBaseFinalizeFunc) NULL, /* base finalize */
-      (GClassInitFunc) gail_entry_class_init, /* class init */
-      (GClassFinalizeFunc) NULL, /* class finalize */
-      NULL, /* class data */
-      sizeof (GailEntry), /* instance size */
-      0, /* nb preallocs */
-      (GInstanceInitFunc) gail_entry_object_init, /* instance init */
-      NULL /* value table */
-    };
+      static const GTypeInfo tinfo =
+      {
+        sizeof (GailEntryClass),
+        (GBaseInitFunc) NULL, /* base init */
+        (GBaseFinalizeFunc) NULL, /* base finalize */
+        (GClassInitFunc) gail_entry_class_init, /* class init */
+        (GClassFinalizeFunc) NULL, /* class finalize */
+        NULL, /* class data */
+        sizeof (GailEntry), /* instance size */
+        0, /* nb preallocs */
+        (GInstanceInitFunc) gail_entry_object_init, /* instance init */
+        NULL /* value table */
+      };
 
-    static const GInterfaceInfo atk_editable_text_info =
-    {
+      static const GInterfaceInfo atk_editable_text_info =
+      {
         (GInterfaceInitFunc) atk_editable_text_interface_init,
         (GInterfaceFinalizeFunc) NULL,
         NULL
-    };
+      };
 
-    static const GInterfaceInfo atk_text_info =
-    {
+      static const GInterfaceInfo atk_text_info =
+      {
         (GInterfaceInitFunc) atk_text_interface_init,
         (GInterfaceFinalizeFunc) NULL,
         NULL
-    };
+      };
 
-    type = g_type_register_static (GAIL_TYPE_WIDGET,
-                                   "GailEntry", &tinfo, 0);
-    g_type_add_interface_static (type, ATK_TYPE_EDITABLE_TEXT,
-                                 &atk_editable_text_info);
-    g_type_add_interface_static (type, ATK_TYPE_TEXT,
-                                 &atk_text_info);
+      type = g_type_register_static (GAIL_TYPE_WIDGET,
+                                     "GailEntry", &tinfo, 0);
+      g_type_add_interface_static (type, ATK_TYPE_EDITABLE_TEXT,
+                                   &atk_editable_text_info);
+      g_type_add_interface_static (type, ATK_TYPE_TEXT,
+                                   &atk_text_info);
 
-
-  }
+    }
   return type;
 }
 
@@ -217,6 +217,8 @@ gail_entry_object_init (GailEntry *entry)
 {
   entry->texthelper = NULL;
   entry->signal_name = NULL;
+  entry->cursor_position = 0;
+  entry->selection_bound = 0;
 }
 
 static void
@@ -228,18 +230,18 @@ gail_entry_real_init (GailWidget *widget,
 
   parent_class->init (widget, gtk_widget);
 
-  GAIL_ENTRY(widget)->texthelper = gail_text_helper_new();
+  GAIL_ENTRY (widget)->texthelper = gail_text_helper_new ();
   
   entry = GTK_ENTRY (gtk_widget);
-  entry_text = gtk_entry_get_text(entry);
-  gail_text_helper_text_setup(GAIL_ENTRY(widget)->texthelper, entry_text);
+  entry_text = gtk_entry_get_text (entry);
+  gail_text_helper_text_setup (GAIL_ENTRY(widget)->texthelper, entry_text);
 
   /* Set up signal callbacks */
-  g_signal_connect(G_OBJECT(gtk_widget), "insert-text",
+  g_signal_connect (G_OBJECT (gtk_widget), "insert-text",
 	G_CALLBACK (_gail_entry_insert_text_cb), NULL);
-  g_signal_connect(G_OBJECT(gtk_widget), "delete-text",
+  g_signal_connect (G_OBJECT (gtk_widget), "delete-text",
 	G_CALLBACK (_gail_entry_delete_text_cb), NULL);
-  g_signal_connect(G_OBJECT(gtk_widget), "changed",
+  g_signal_connect (G_OBJECT (gtk_widget), "changed",
 	G_CALLBACK (_gail_entry_changed_cb), NULL);
 }
 
@@ -249,57 +251,35 @@ gail_entry_real_notify_gtk (GObject		*obj,
 {
   GtkWidget *widget = GTK_WIDGET (obj);
   AtkObject* atk_obj = gtk_widget_get_accessible (widget);
+  GtkEntry* gtk_entry = GTK_ENTRY (widget);
+  GailEntry* entry = GAIL_ENTRY (atk_obj);
 
-  if (strcmp (pspec->name, "text-position") == 0)
-  {
-    if (atk_obj->name == NULL)
+  if (strcmp (pspec->name, "cursor-position") == 0)
     {
-       GtkEntry *entry;
-       gint caret_pos;
-
-       entry = GTK_ENTRY (widget);
-
+      if (check_for_selection_change (entry, gtk_entry))
+        g_signal_emit_by_name (atk_obj, "selection_changed");
       /*
        * The entry cursor position has moved so generate the signal.
        */
-       caret_pos = gtk_editable_get_position (GTK_EDITABLE (entry));
-       g_signal_emit_by_name (G_OBJECT(atk_obj), "text-caret-moved", caret_pos);
+      g_signal_emit_by_name (atk_obj, "text_caret_moved", 
+                             entry->cursor_position);
+    }
+  else if (strcmp (pspec->name, "selection-bound") == 0)
+    {
+      if (check_for_selection_change (entry, gtk_entry))
+        g_signal_emit_by_name (atk_obj, "selection_changed");
     }
-  }
   else if (strcmp (pspec->name, "editable") == 0)
-  {
-    AtkPropertyValues values;
-    GValue *value, val;
-    gboolean state_value;
-
-    memset (&values.old_value, 0, sizeof (GValue));
-    memset (&values.new_value, 0, sizeof (GValue));
-  
-    value = &val;
-  
-    memset (value, 0, sizeof (value));
-
-    g_value_init (value, G_TYPE_BOOLEAN);
-    g_object_get_property (obj, "editable", value);
-    state_value = g_value_get_boolean (value);
-    if (state_value)
     {
-      g_value_init (&values.new_value, G_TYPE_INT);
-      g_value_set_int (&values.new_value, ATK_STATE_EDITABLE);
+      GValue value = { 0, };
+
+      g_value_init (&value, G_TYPE_BOOLEAN);
+      g_object_get_property (obj, "editable", &value);
+      atk_object_notify_state_change (atk_obj, ATK_STATE_EDITABLE,
+                                               g_value_get_boolean (&value));
     }
-    else
-    {
-      g_value_init (&values.old_value, G_TYPE_INT);
-      g_value_set_int (&values.old_value, ATK_STATE_EDITABLE);
-    }
-    values.property_name = "accessible-state";
-    g_signal_emit_by_name (G_OBJECT (atk_obj), "property-change",
-                   &values, NULL);
-  }
   else
-  {
     parent_class->notify_gtk (obj, pspec);
-  }
 }
 
 static void
@@ -327,9 +307,9 @@ gail_entry_new (GtkWidget *widget)
 
   accessible = GTK_ACCESSIBLE (object);
   if (GTK_ENTRY (widget)->visible)
-    ATK_OBJECT(accessible)->role = ATK_ROLE_TEXT;
+    ATK_OBJECT (accessible)->role = ATK_ROLE_TEXT;
   else
-    ATK_OBJECT(accessible)->role = ATK_ROLE_PASSWORD_TEXT;
+    ATK_OBJECT (accessible)->role = ATK_ROLE_PASSWORD_TEXT;
 
   return accessible;
 }
@@ -342,10 +322,9 @@ gail_entry_get_index_in_parent (AtkObject *accessible)
    * otherwise do the normal thing.
    */
   if (accessible->accessible_parent)
-  {
     if (GAIL_IS_COMBO (accessible->accessible_parent))
       return 1;
-  }
+
   return ATK_OBJECT_CLASS (parent_class)->get_index_in_parent (accessible);
 }
 
@@ -356,8 +335,7 @@ gail_entry_ref_state_set (AtkObject *accessible)
 {
   AtkStateSet *state_set;
   GtkEntry *entry;
-  GValue *value, val;
-  gboolean state_value;
+  GValue value = { 0, };
   GtkWidget *widget;
 
   state_set = ATK_OBJECT_CLASS (parent_class)->ref_state_set (accessible);
@@ -368,13 +346,9 @@ gail_entry_ref_state_set (AtkObject *accessible)
 
   entry = GTK_ENTRY (widget);
 
-  value = &val;
-  memset (value, 0, sizeof (value));
-
-  g_value_init (value, G_TYPE_BOOLEAN);
-  g_object_get_property (G_OBJECT (entry), "editable", value);
-  state_value = g_value_get_boolean (value);
-  if (state_value)
+  g_value_init (&value, G_TYPE_BOOLEAN);
+  g_object_get_property (G_OBJECT (entry), "editable", &value);
+  if (g_value_get_boolean (&value))
     atk_state_set_add_state (state_set, ATK_STATE_EDITABLE);
   atk_state_set_add_state (state_set, ATK_STATE_SINGLE_LINE);
 
@@ -401,7 +375,8 @@ atk_text_interface_init (AtkTextIface *iface)
   iface->add_selection = gail_entry_add_selection;
   iface->remove_selection = gail_entry_remove_selection;
   iface->set_selection = gail_entry_set_selection;
-  iface->ref_run_attributes = gail_entry_ref_run_attributes;
+  iface->get_run_attributes = gail_entry_get_run_attributes;
+  iface->get_default_attributes = gail_entry_get_default_attributes;
   iface->get_character_extents = gail_entry_get_character_extents;
   iface->get_offset_at_point = gail_entry_get_offset_at_point;
 }
@@ -416,10 +391,8 @@ gail_entry_get_text (AtkText *text,
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return NULL;
-  }
 
   entry = GTK_ENTRY (widget);
   
@@ -435,71 +408,41 @@ gail_entry_get_text_before_offset (AtkText	    *text,
 {
   GtkWidget *widget;
   GtkEntry *entry;
-  const gchar* entry_text;
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return NULL;
-  }
 
   /* Get Entry */
   entry = GTK_ENTRY (widget);
-  entry_text = gtk_entry_get_text(entry);
 
-  if (entry_text == NULL)
-    return 0;
-
-  if (boundary_type == ATK_TEXT_BOUNDARY_CHAR)
-  {
-    return gail_text_helper_boundary_char (GAIL_ENTRY(text)->texthelper, 
-                                           offset, start_offset, end_offset);
-  }
-  else
-  {
-    return gail_text_helper_switch_boundary_type (GAIL_ENTRY(text)->texthelper,
-                          gtk_entry_get_layout(entry), GAIL_BEFORE_OFFSET, 
+  return gail_text_helper_get_text (GAIL_ENTRY (text)->texthelper,
+                          gtk_entry_get_layout (entry), GAIL_BEFORE_OFFSET, 
                           boundary_type, offset, start_offset, end_offset);
-  } 
 }
 
 static gchar*
-gail_entry_get_text_at_offset	  (AtkText	    *text,
-				   gint		    offset,
-				   AtkTextBoundary  boundary_type,
-				   gint		    *start_offset,
-				   gint		    *end_offset)
+gail_entry_get_text_at_offset (AtkText          *text,
+                               gint             offset,
+                               AtkTextBoundary  boundary_type,
+                               gint             *start_offset,
+                               gint             *end_offset)
 {
   GtkWidget *widget;
   GtkEntry *entry;
-  const gchar* entry_text;
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return NULL;
-  }
 
   /* Get Entry */
   entry = GTK_ENTRY (widget);
-  entry_text = gtk_entry_get_text(entry);
 
-  if (entry_text == NULL)
-    return 0;
-
-  if (boundary_type == ATK_TEXT_BOUNDARY_CHAR)
-  {
-    return gail_text_helper_boundary_char (GAIL_ENTRY(text)->texthelper, 
-                                           offset, start_offset, end_offset); 
-  }
-  else
-  {
-    return gail_text_helper_switch_boundary_type (GAIL_ENTRY(text)->texthelper,
-                            gtk_entry_get_layout(entry), GAIL_AT_OFFSET, 
+  return gail_text_helper_get_text (GAIL_ENTRY (text)->texthelper,
+                            gtk_entry_get_layout (entry), GAIL_AT_OFFSET, 
                             boundary_type, offset, start_offset, end_offset);
-  }
 }
 
 static gchar*
@@ -511,33 +454,18 @@ gail_entry_get_text_after_offset  (AtkText	    *text,
 {
   GtkWidget *widget;
   GtkEntry *entry;
-  const gchar* entry_text;
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return NULL;
-  }
 
   /* Get Entry */
   entry = GTK_ENTRY (widget);
-  entry_text = gtk_entry_get_text(entry);
 
-  if (entry_text == NULL)
-    return 0;
-
-  if (boundary_type == ATK_TEXT_BOUNDARY_CHAR)
-  {
-    return gail_text_helper_boundary_char (GAIL_ENTRY(text)->texthelper, 
-                                           offset, start_offset, end_offset);
-  }
-  else
-  {
-    return gail_text_helper_switch_boundary_type (GAIL_ENTRY(text)->texthelper,
-                           gtk_entry_get_layout(entry), GAIL_AFTER_OFFSET, 
+  return gail_text_helper_get_text (GAIL_ENTRY (text)->texthelper,
+                           gtk_entry_get_layout (entry), GAIL_AFTER_OFFSET, 
                            boundary_type, offset, start_offset, end_offset);
-  }
 }
 
 static gint
@@ -548,13 +476,11 @@ gail_entry_get_character_count (AtkText *text)
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return 0;
-  }
 
   entry = GTK_ENTRY (widget);
-  return(g_utf8_strlen(gtk_entry_get_text(entry), -1));
+  return g_utf8_strlen (gtk_entry_get_text (entry), -1);
 }
 
 static gint
@@ -565,14 +491,12 @@ gail_entry_get_caret_offset (AtkText *text)
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return 0;
-  }
 
   entry = GTK_ENTRY (widget);
 
-  return (gtk_editable_get_position (GTK_EDITABLE (entry)));
+  return gtk_editable_get_position (GTK_EDITABLE (entry));
 }
 
 static gboolean
@@ -583,10 +507,8 @@ gail_entry_set_caret_offset (AtkText *text, gint offset)
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return FALSE;
-  }
 
   entry = GTK_ENTRY (widget);
 
@@ -595,14 +517,65 @@ gail_entry_set_caret_offset (AtkText *text, gint offset)
 }
 
 static AtkAttributeSet*
-gail_entry_ref_run_attributes (AtkText *text,
-			       gint offset,
-                               gint *start_offset,
-                               gint *end_offset)
+gail_entry_get_run_attributes (AtkText *text,
+			       gint    offset,
+                               gint    *start_offset,
+                               gint    *end_offset)
 {
-  *start_offset = 0;
-  *end_offset = gail_entry_get_character_count (text);
-  return NULL;
+  GtkWidget *widget;
+  GtkEntry *entry;
+  AtkAttributeSet *at_set = NULL;
+  GtkTextDirection dir;
+
+  widget = GTK_ACCESSIBLE (text)->widget;
+  if (widget == NULL)
+    /* State is defunct */
+    return NULL;
+
+  entry = GTK_ENTRY (widget);
+ 
+  dir = gtk_widget_get_direction (widget);
+  if (dir == GTK_TEXT_DIR_RTL)
+    {
+      at_set = gail_text_helper_add_attribute (at_set,
+                                               ATK_TEXT_ATTR_DIRECTION,
+       g_strdup (atk_attribute_get_value (ATK_TEXT_ATTR_DIRECTION, dir)));
+    }
+
+  at_set = gail_text_helper_get_run_attributes (at_set,
+                                                gtk_entry_get_layout (entry),
+                                                entry->text,
+                                                offset,
+                                                start_offset,
+                                                end_offset);
+  return at_set;
+}
+
+static AtkAttributeSet*
+gail_entry_get_default_attributes (AtkText *text)
+{
+  GtkWidget *widget;
+  GtkEntry *entry;
+  AtkAttributeSet *at_set = NULL;
+
+  widget = GTK_ACCESSIBLE (text)->widget;
+  if (widget == NULL)
+    /* State is defunct */
+    return NULL;
+
+  entry = GTK_ENTRY (widget);
+
+  at_set = gail_text_helper_add_attribute (at_set,
+                                           ATK_TEXT_ATTR_JUSTIFICATION,
+     g_strdup (atk_attribute_get_value (ATK_TEXT_ATTR_JUSTIFICATION, 0)));
+  at_set = gail_text_helper_add_attribute (at_set,
+                                           ATK_TEXT_ATTR_WRAP_MODE,
+              g_strdup (atk_attribute_get_value (ATK_TEXT_ATTR_WRAP_MODE, 0)));
+  at_set = gail_text_helper_get_default_attributes (at_set,
+                                                    gtk_entry_get_layout (entry)
+,
+                                                    widget);
+  return at_set;
 }
   
 static void
@@ -621,18 +594,18 @@ gail_entry_get_character_extents (AtkText *text,
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return;
-  }
+
   entry = GTK_ENTRY (widget);
 
-  gtk_entry_get_layout_offsets(entry, &x_layout, &y_layout);
-  index = g_utf8_offset_to_pointer(entry->text, offset) - entry->text;
-  cursor_index = g_utf8_offset_to_pointer (entry->text, entry->current_pos) - entry->text;
+  gtk_entry_get_layout_offsets (entry, &x_layout, &y_layout);
+  index = g_utf8_offset_to_pointer (entry->text, offset) - entry->text;
+  cursor_index = g_utf8_offset_to_pointer (entry->text, entry->current_pos) - 
+                          entry->text;
   if (index > cursor_index)
     index += entry->preedit_length;
-  pango_layout_index_to_pos(gtk_entry_get_layout(entry), index, &char_rect);
+  pango_layout_index_to_pos (gtk_entry_get_layout(entry), index, &char_rect);
  
   gail_text_helper_get_extents_from_pango_rectangle (widget, &char_rect, 
                         x_layout, y_layout, x, y, width, height, coords);
@@ -650,31 +623,31 @@ gail_entry_get_offset_at_point (AtkText *text,
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return -1;
-  }
+
   entry = GTK_ENTRY (widget);
   
-  gtk_entry_get_layout_offsets(entry, &x_layout, &y_layout);
+  gtk_entry_get_layout_offsets (entry, &x_layout, &y_layout);
   
   index = gail_text_helper_get_index_at_point_in_layout (widget, 
                gtk_entry_get_layout(entry), x_layout, y_layout, x, y, coords);
   if (index == -1)
     return -1;  
   else
-  {
-    cursor_index = g_utf8_offset_to_pointer (entry->text, entry->current_pos) -
+    {
+      cursor_index = g_utf8_offset_to_pointer (entry->text, 
+                                               entry->current_pos) -
                                              entry->text;
-    if (index >= cursor_index && entry->preedit_length)
-      {
-        if (index >= cursor_index + entry->preedit_length)
-          index -= entry->preedit_length;
-        else
-          index = cursor_index;
-      }
-    return g_utf8_pointer_to_offset(entry->text, entry->text + index);
-  }
+      if (index >= cursor_index && entry->preedit_length)
+        {
+          if (index >= cursor_index + entry->preedit_length)
+            index -= entry->preedit_length;
+          else
+            index = cursor_index;
+        }
+      return g_utf8_pointer_to_offset (entry->text, entry->text + index);
+    }
 }
 
 /* atkeditable.h */
@@ -690,16 +663,7 @@ atk_editable_text_interface_init (AtkEditableTextIface *iface)
   iface->cut_text = gail_entry_cut_text;
   iface->delete_text = gail_entry_delete_text;
   iface->paste_text = gail_entry_paste_text;
-  iface->set_run_attributes = gail_entry_set_run_attributes;
-}
-
-static gboolean 
-gail_entry_set_run_attributes (AtkEditableText *text,
-                               AtkAttributeSet *attrib_set,
-                               gint start_offset,
- 			       gint end_offset)
-{
-  return FALSE;
+  iface->set_run_attributes = NULL;;
 }
 
 static gint
@@ -711,10 +675,8 @@ gail_entry_get_n_selections (AtkText              *text)
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return -1;
-  }
 
   entry = GTK_ENTRY (widget);
 
@@ -722,9 +684,9 @@ gail_entry_get_n_selections (AtkText              *text)
                                      &select_end);
 
   if (select_start != select_end)
-     return(1);
+    return 1;
   else
-     return(0);
+    return 0;
 }
 
 static gchar*
@@ -738,10 +700,8 @@ gail_entry_get_selection (AtkText *text,
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return NULL;
-  }
 
  /* Only let the user get the selection if one is set, and if the
   * selection_num is 0.
@@ -769,10 +729,8 @@ gail_entry_add_selection (AtkText *text,
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return FALSE;
-  }
 
   entry = GTK_ENTRY (widget);
 
@@ -783,14 +741,12 @@ gail_entry_add_selection (AtkText *text,
   * since GtkEntry only supports one selected region.
   */
   if (select_start == select_end)
-  {
-     gtk_editable_select_region (GTK_EDITABLE (entry), start_pos, end_pos);
-     return TRUE;
-  }
+    {
+       gtk_editable_select_region (GTK_EDITABLE (entry), start_pos, end_pos);
+       return TRUE;
+    }
   else
-  {
-     return FALSE;
-  }
+   return FALSE;
 }
 
 static gboolean
@@ -803,10 +759,8 @@ gail_entry_remove_selection (AtkText *text,
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return FALSE;
-  }
 
   if (selection_num != 0)
      return FALSE;
@@ -816,18 +770,16 @@ gail_entry_remove_selection (AtkText *text,
                                      &select_end);
 
   if (select_start != select_end)
-  {
-   /* Setting the start & end of the selected region to the caret position
-    * turns off the selection.
-    */
-    caret_pos = gtk_editable_get_position (GTK_EDITABLE (entry));
-    gtk_editable_select_region (GTK_EDITABLE (entry), caret_pos, caret_pos);
-    return TRUE;
-  }
+    {
+     /* Setting the start & end of the selected region to the caret position
+      * turns off the selection.
+      */
+      caret_pos = gtk_editable_get_position (GTK_EDITABLE (entry));
+      gtk_editable_select_region (GTK_EDITABLE (entry), caret_pos, caret_pos);
+      return TRUE;
+    }
   else
-  {
     return FALSE;
-  }
 }
 
 static gboolean
@@ -842,10 +794,8 @@ gail_entry_set_selection (AtkText *text,
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return FALSE;
-  }
 
  /* Only let the user move the selection if one is set, and if the
   * selection_num is 0
@@ -859,14 +809,12 @@ gail_entry_set_selection (AtkText *text,
                                      &select_end);
 
   if (select_start != select_end)
-  {
-    gtk_editable_select_region (GTK_EDITABLE (entry), start_pos, end_pos);
-    return TRUE;
-  }
+    {
+      gtk_editable_select_region (GTK_EDITABLE (entry), start_pos, end_pos);
+      return TRUE;
+    }
   else
-  {
     return FALSE;
-  }
 }
 
 static void
@@ -878,10 +826,8 @@ gail_entry_set_text_contents (AtkEditableText *text,
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return;
-  }
 
   entry = GTK_ENTRY (widget);
 
@@ -899,10 +845,8 @@ gail_entry_insert_text (AtkEditableText *text,
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return;
-  }
 
   entry = GTK_ENTRY (widget);
 
@@ -920,10 +864,8 @@ gail_entry_copy_text   (AtkEditableText *text,
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return;
-  }
 
   entry = GTK_ENTRY (widget);
   str = gtk_editable_get_chars (GTK_EDITABLE (entry), start_pos, end_pos);
@@ -941,15 +883,13 @@ gail_entry_cut_text (AtkEditableText *text,
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return;
-  }
 
   entry = GTK_ENTRY (widget);
   str = gtk_editable_get_chars (GTK_EDITABLE (entry), start_pos, end_pos);
-  gtk_clipboard_set_text(gtk_clipboard_get(GDK_NONE), str, -1);
-  gtk_editable_delete_text(GTK_EDITABLE(entry), start_pos, end_pos);
+  gtk_clipboard_set_text (gtk_clipboard_get (GDK_NONE), str, -1);
+  gtk_editable_delete_text (GTK_EDITABLE (entry), start_pos, end_pos);
 }
 
 static void
@@ -962,10 +902,8 @@ gail_entry_delete_text (AtkEditableText *text,
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return;
-  }
 
   entry = GTK_ENTRY (widget);
 
@@ -981,10 +919,8 @@ gail_entry_paste_text (AtkEditableText *text,
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return;
-  }
 
   paste_struct.entry = GTK_ENTRY (widget);
   paste_struct.position = position;
@@ -1002,10 +938,8 @@ gail_entry_paste_received (GtkClipboard *clipboard,
   GailEntryPaste* paste_struct = (GailEntryPaste *)data;
 
   if (text)
-  {
-    gtk_editable_insert_text (GTK_EDITABLE(paste_struct->entry), text, -1,
+    gtk_editable_insert_text (GTK_EDITABLE (paste_struct->entry), text, -1,
        &(paste_struct->position));
-  }
 
   g_object_unref (paste_struct->entry);
 }
@@ -1023,19 +957,15 @@ static void _gail_entry_insert_text_cb (GtkEntry *entry,
   AtkObject *accessible;
   GailEntry *gail_entry;
   gint *position = (gint *) arg3;
-  const gchar* entry_text;
-  
-  entry_text = gtk_entry_get_text(entry);
 
-  accessible = gtk_widget_get_accessible(GTK_WIDGET(entry));
+  accessible = gtk_widget_get_accessible (GTK_WIDGET (entry));
   gail_entry = GAIL_ENTRY (accessible);
-  gail_text_helper_text_setup(gail_entry->texthelper, entry_text);
   if (!gail_entry->signal_name)
-  {
-    gail_entry->signal_name = "text-changed::insert";
-    gail_entry->position = *position;
-    gail_entry->length = arg2;
-  }
+    {
+      gail_entry->signal_name = "text_changed::insert";
+      gail_entry->position = *position;
+      gail_entry->length = arg2;
+    }
   /*
    * The signal will be emitted when the changed signal is received
    */
@@ -1052,14 +982,12 @@ gail_entry_get_character_at_offset (AtkText	         *text,
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
-  {
     /* State is defunct */
     return '\0';
-  }
 
   entry = GTK_ENTRY (widget);
-  string = gtk_editable_get_chars(GTK_EDITABLE (entry), 0, -1);
-  index = g_utf8_offset_to_pointer(string, offset);
+  string = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
+  index = g_utf8_offset_to_pointer (string, offset);
   g_free(string);
 
   return g_utf8_get_char(index);
@@ -1073,15 +1001,10 @@ static void _gail_entry_delete_text_cb (GtkEntry *entry,
                                         gint      arg2)
 {
   AtkObject *accessible;
-  GailEntry *gail_entry;
 
-  accessible = gtk_widget_get_accessible(GTK_WIDGET(entry));
-  gail_entry = GAIL_ENTRY (accessible);
+  accessible = gtk_widget_get_accessible (GTK_WIDGET (entry));
 
-  gail_text_helper_text_setup(gail_entry->texthelper, "");
-
-  g_signal_emit_by_name(G_OBJECT(accessible), "text-changed::delete",
-     arg1, arg2 - arg1);
+  g_signal_emit_by_name (accessible, "text_changed::delete", arg1, arg2 - arg1);
 }
 
 static void
@@ -1090,17 +1013,53 @@ _gail_entry_changed_cb (GtkEntry *entry)
   AtkObject *accessible;
   GailEntry *gail_entry;
 
-  accessible = gtk_widget_get_accessible(GTK_WIDGET(entry));
+  accessible = gtk_widget_get_accessible (GTK_WIDGET (entry));
 
   gail_entry = GAIL_ENTRY (accessible);
 
   if (gail_entry->signal_name)
-  {
-    g_signal_emit_by_name(G_OBJECT(accessible), 
-                          gail_entry->signal_name,
-                          gail_entry->position,
-                          gail_entry->length);
-    gail_entry->signal_name = NULL;
-  }
-  g_object_notify (G_OBJECT (accessible), "accessible-text");
+    {
+      g_signal_emit_by_name (accessible, 
+                             gail_entry->signal_name,
+                             gail_entry->position,
+                             gail_entry->length);
+      gail_entry->signal_name = NULL;
+    }
+  /*
+   * Update the text in the GtkTextBuffer
+   */
+  gail_text_helper_text_setup (gail_entry->texthelper, 
+                               gtk_entry_get_text(entry));
+}
+
+static gboolean 
+check_for_selection_change (GailEntry   *entry,
+                            GtkEntry    *gail_entry)
+{
+  gint pos;
+  gboolean ret_val = FALSE;
+ 
+  pos = gtk_editable_get_position (GTK_EDITABLE (gail_entry));
+  if (gail_entry->current_pos != gail_entry->selection_bound)
+    {
+      if (gail_entry->current_pos != entry->cursor_position ||
+          gail_entry->selection_bound != entry->selection_bound)
+        /*
+         * This check is here as this function can be called
+         * for notification of selection_bound or current_pos.
+         * The values of current_pos  selection_bound may be the same 
+         * for both notifications and we only want to generate one
+         * selection_changed signal.
+         */
+        ret_val = TRUE;
+    }
+  else 
+    {
+      /* We had a selection */
+      ret_val = (entry->cursor_position != entry->selection_bound);
+    }
+  entry->cursor_position = gail_entry->current_pos;
+  entry->selection_bound = gail_entry->selection_bound;
+
+  return ret_val;
 }
