@@ -289,7 +289,7 @@ find_object_by_type (AtkObject *obj, gchar *type)
  * Keeps a static GPtrArray of objects that have been passed into this
  * function. 
  *
- * Returns: TRUE if @obj has not been passed into this function before
+ * Returns: TRUE if @obj has been passed into this function before
  * and FALSE otherwise.
  */
 gboolean
@@ -324,12 +324,11 @@ already_accessed_atk_object (AtkObject *obj)
     g_ptr_array_add (obj_array, obj);
   }
 
-  /* Returning !found */
-  return(!found);
+  return found;
 }
 
 /* Global list of ATK Role Names */
-#define ATK_NUM_ROLES	68
+#define ATK_NUM_ROLES	69
 static gchar g_role_names[ATK_NUM_ROLES][100] =
 {
   "ATK_ROLE_INVALID",
@@ -391,6 +390,7 @@ static gchar g_role_names[ATK_NUM_ROLES][100] =
   "ATK_ROLE_TABLE_COLUMN_HEADER",
   "ATK_ROLE_TABLE_ROW_HEADER",
   "ATK_ROLE_TEAR_OFF_MENU_ITEM",
+  "ATK_ROLE_TERMINAL",
   "ATK_ROLE_TEXT",
   "ATK_ROLE_TOGGLE_BUTTON",
   "ATK_ROLE_TOOL_BAR",
@@ -695,7 +695,7 @@ _create_select_tests_window(AtkObject *obj, TLruntest runtest, OutputWindow **ou
       
        /* Setup Layout */
        md[window_no]->vbox = gtk_vbox_new(TRUE, 0);
-       md[window_no]->button = gtk_button_new_with_label("Run Tests");
+       md[window_no]->button = gtk_button_new_with_mnemonic("_Run Tests");
        hbuttonbox = gtk_hbutton_box_new();
        gtk_button_box_set_layout(GTK_BUTTON_BOX(hbuttonbox),
          GTK_BUTTONBOX_SPREAD);
