@@ -85,7 +85,9 @@ gint setup_gui(AtkObject *obj, TLruntest test)
 
   paramnames[0] = "offset";
   defaults[0] = "36";
-  add_test(window, "atk_text_ref_run_attributes", 1, paramnames, defaults);
+  add_test(window, "atk_text_get_run_attributes", 1, paramnames, defaults);
+
+  add_test(window, "atk_text_get_default_attributes", 0, paramnames, defaults);
 
   paramnames[0] = "offset";
   paramnames[1] = "coord mode";
@@ -150,7 +152,7 @@ gint setup_gui(AtkObject *obj, TLruntest test)
  **/
 void add_handlers(AtkObject *obj)
 {
-  if (already_accessed_atk_object(obj) == TRUE)
+  if (!already_accessed_atk_object(obj))
   {
     /* Set up signal handlers */
 
@@ -479,23 +481,53 @@ runtest(AtkObject *obj, gint win_val)
       set_output_buffer(output);
     }
 
-    if (strcmp(testsOn[i], "atk_text_ref_run_attributes") == 0)
+    if (strcmp(testsOn[i], "atk_text_get_run_attributes") == 0)
     {
       gint test_int;
-      param_string1 = get_arg_of_func(win_val, "atk_text_ref_run_attributes", "offset");
+      param_string1 = get_arg_of_func(win_val, "atk_text_get_run_attributes", "offset");
       test_int = string_to_int(param_string1);
-      attrib = atk_text_ref_run_attributes(ATK_TEXT(obj), test_int, &start, &end);
-      sprintf(output, "ref_run_attributes at offset %i:\nStart: %i, End: %i\n", test_int,
+      attrib = atk_text_get_run_attributes(ATK_TEXT(obj), test_int, &start, &end);
+      sprintf(output, "get_run_attributes at offset %i:\nStart: %i, End: %i\n", test_int,
         start, end);
       set_output_buffer(output);
       if (attrib != NULL) {
-        for (index = 0; index < g_slist_length(attrib); index++)
+        GSList *node;
+        index = 0;
+        node = attrib;
+        while (node != NULL)
         {
+          AtkAttribute* att = node->data;
+
           sprintf(output, "List index: %i, Name: %s, Value: %s\n", index,
-            ((AtkAttribute*)g_slist_nth(attrib, index)->data)->name,
-            ((AtkAttribute*)g_slist_nth(attrib, index)->data)->value);
+            att->name, att->value);
           set_output_buffer(output);
+          node = node->next;
+          index++;
         }
+        atk_attribute_set_free (attrib);
+      }
+    }
+
+    if (strcmp(testsOn[i], "atk_text_get_default_attributes") == 0)
+    {
+      attrib = atk_text_get_default_attributes(ATK_TEXT(obj));
+      sprintf(output, "get_default_attributes\n");
+      set_output_buffer(output);
+      if (attrib != NULL) {
+        GSList *node;
+        index = 0;
+        node = attrib;
+        while (node != NULL)
+        {
+          AtkAttribute* att = node->data;
+
+          sprintf(output, "List index: %i, Name: %s, Value: %s\n", index,
+            att->name, att->value);
+          set_output_buffer(output);
+          node = node->next;
+          index++;
+        }
+        atk_attribute_set_free (attrib);
       }
     }
  
@@ -669,7 +701,7 @@ void _run_offset_test(AtkObject * obj, char * type, gint offset,
        offset, boundary, &startOffset, &endOffset);
 
   if (text == NULL)
-    text = strdup (default_val);
+    text = g_strdup (default_val);
     
   if (boundary == ATK_TEXT_BOUNDARY_CHAR)
     sprintf(output, "\n|%s| Text |%s|  Boundary |BOUNDARY_CHAR|\n",
